@@ -121,8 +121,6 @@ inline void health_alarm_log_save(RRDHOST *host, ALARM_ENTRY *ae) {
 }
 
 inline ssize_t health_alarm_log_read(RRDHOST *host, FILE *fp, const char *filename) {
-    static uint32_t max_unique_id = 0, max_alarm_id = 0;
-
     errno = 0;
 
     char *s, *buf = mallocz(65536 + 1);
@@ -271,7 +269,7 @@ inline ssize_t health_alarm_log_read(RRDHOST *host, FILE *fp, const char *filena
             ae->new_value   = str2l(pointers[25]);
             ae->old_value   = str2l(pointers[26]);
 
-            static char value_string[100 + 1];
+            char value_string[100 + 1];
             freez(ae->old_value_string);
             freez(ae->new_value_string);
             ae->old_value_string = strdupz(format_value_and_unit(value_string, 100, ae->old_value, ae->units, -1));
@@ -285,11 +283,11 @@ inline ssize_t health_alarm_log_read(RRDHOST *host, FILE *fp, const char *filena
             }
             else updated++;
 
-            if(unlikely(ae->unique_id > max_unique_id))
-                max_unique_id = ae->unique_id;
+            if(unlikely(ae->unique_id > host->health_max_unique_id))
+                host->health_max_unique_id = ae->unique_id;
 
-            if(unlikely(ae->alarm_id >= max_alarm_id))
-                max_alarm_id = ae->alarm_id;
+            if(unlikely(ae->alarm_id >= host->health_max_alarm_id))
+                host->health_max_alarm_id = ae->alarm_id;
         }
         else {
             error("HEALTH [%s]: line %zu of file '%s' is invalid (unrecognized entry type '%s').", host->hostname, line, filename, pointers[0]);
@@ -301,11 +299,11 @@ inline ssize_t health_alarm_log_read(RRDHOST *host, FILE *fp, const char *filena
 
     freez(buf);
 
-    if(!max_unique_id) max_unique_id = (uint32_t)now_realtime_sec();
-    if(!max_alarm_id)  max_alarm_id  = (uint32_t)now_realtime_sec();
+    if(!host->health_max_unique_id) host->health_max_unique_id = (uint32_t)now_realtime_sec();
+    if(!host->health_max_alarm_id)  host->health_max_alarm_id  = (uint32_t)now_realtime_sec();
 
-    host->health_log.next_log_id = max_unique_id + 1;
-    host->health_log.next_alarm_id = max_alarm_id + 1;
+    host->health_log.next_log_id = host->health_max_unique_id + 1;
+    host->health_log.next_alarm_id = host->health_max_alarm_id + 1;
 
     debug(D_HEALTH, "HEALTH [%s]: loaded file '%s' with %zd new alarm entries, updated %zd alarms, errors %zd entries, duplicate %zd", host->hostname, filename, loaded, updated, errored, duplicate);
     return loaded;
@@ -388,7 +386,7 @@ inline void health_alarm_log(
     ae->old_value = old_value;
     ae->new_value = new_value;
 
-    static char value_string[100 + 1];
+    char value_string[100 + 1];
     ae->old_value_string = strdupz(format_value_and_unit(value_string, 100, ae->old_value, ae->units, -1));
     ae->new_value_string = strdupz(format_value_and_unit(value_string, 100, ae->new_value, ae->units, -1));
 
