@@ -32,4 +32,8 @@ extern int get_numa_node_count(void);
 // metrics that need to be shared among data collectors
 extern unsigned long long tcpext_TCPSynRetrans;
 
+// netdev renames
+extern void netdev_rename_device_add(const char *host_device, const char *container_device, const char *container_name);
+extern void netdev_rename_device_del(const char *host_device);
+
 #endif /* NETDATA_PLUGIN_PROC_H */
