@@ -816,7 +816,7 @@ inline int web_client_api_request_v1_registry(RRDHOST *host, struct web_client *
 #endif /* NETDATA_INTERNAL_CHECKS */
     }
 
-    if(respect_web_browser_do_not_track_policy && w->donottrack) {
+    if(respect_web_browser_do_not_track_policy && web_client_has_donottrack(w)) {
         buffer_flush(w->response.data);
         buffer_sprintf(w->response.data, "Your web browser is sending 'DNT: 1' (Do Not Track). The registry requires persistent cookies on your browser to work.");
         return 400;
@@ -853,19 +853,19 @@ inline int web_client_api_request_v1_registry(RRDHOST *host, struct web_client *
 
     switch(action) {
         case 'A':
-            w->tracking_required = 1;
+            web_client_enable_tracking_required(w);
             return registry_request_access_json(host, w, person_guid, machine_guid, machine_url, url_name, now_realtime_sec());
 
         case 'D':
-            w->tracking_required = 1;
+            web_client_enable_tracking_required(w);
             return registry_request_delete_json(host, w, person_guid, machine_guid, machine_url, delete_url, now_realtime_sec());
 
         case 'S':
-            w->tracking_required = 1;
+            web_client_enable_tracking_required(w);
             return registry_request_search_json(host, w, person_guid, machine_guid, machine_url, search_machine_guid, now_realtime_sec());
 
         case 'W':
-            w->tracking_required = 1;
+            web_client_enable_tracking_required(w);
             return registry_request_switch_json(host, w, person_guid, machine_guid, machine_url, to_person_guid, now_realtime_sec());
 
         case 'H':
